@@ -172,7 +172,7 @@ void service_remove(struct btd_service *service)
 	btd_service_unref(service);
 }
 
-int service_accept(struct btd_service *service)
+int service_accept(struct btd_service *service, bool initiator)
 {
 	char addr[18];
 	int err;
@@ -198,6 +198,8 @@ int service_accept(struct btd_service *service)
 		return -ECONNABORTED;
 	}
 
+	service->initiator = initiator;
+
 	err = service->profile->accept(service);
 	if (!err)
 		goto done;
